# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'install_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_InstallDialog(object):
    def setupUi(self, InstallDialog):
        InstallDialog.setObjectName("InstallDialog")
        InstallDialog.resize(338, 267)
        InstallDialog.setWindowTitle("Rare")
        self.install_dialog_layout = QtWidgets.QGridLayout(InstallDialog)
        self.install_dialog_layout.setObjectName("install_dialog_layout")
        self.force_download_check = QtWidgets.QCheckBox(InstallDialog)
        self.force_download_check.setObjectName("force_download_check")
        self.install_dialog_layout.addWidget(self.force_download_check, 3, 1, 1, 1)
        self.install_dir_label = QtWidgets.QLabel(InstallDialog)
        self.install_dir_label.setObjectName("install_dir_label")
        self.install_dialog_layout.addWidget(self.install_dir_label, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        self.download_only_label = QtWidgets.QLabel(InstallDialog)
        self.download_only_label.setObjectName("download_only_label")
        self.install_dialog_layout.addWidget(self.download_only_label, 5, 0, 1, 1, QtCore.Qt.AlignRight)
        self.sdl_list_label = QtWidgets.QLabel(InstallDialog)
        self.sdl_list_label.setObjectName("sdl_list_label")
        self.install_dialog_layout.addWidget(self.sdl_list_label, 6, 0, 1, 1, QtCore.Qt.AlignRight)
        self.install_size_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.install_size_info_label.setFont(font)
        self.install_size_info_label.setObjectName("install_size_info_label")
        self.install_dialog_layout.addWidget(self.install_size_info_label, 8, 1, 1, 2)
        self.install_dialog_label = QtWidgets.QLabel(InstallDialog)
        self.install_dialog_label.setObjectName("install_dialog_label")
        self.install_dialog_layout.addWidget(self.install_dialog_label, 0, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.install_dialog_layout.addItem(spacerItem, 3, 2, 1, 1)
        self.install_size_label = QtWidgets.QLabel(InstallDialog)
        self.install_size_label.setObjectName("install_size_label")
        self.install_dialog_layout.addWidget(self.install_size_label, 8, 0, 1, 1, QtCore.Qt.AlignRight)
        self.download_only_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_only_info_label.setFont(font)
        self.download_only_info_label.setObjectName("download_only_info_label")
        self.install_dialog_layout.addWidget(self.download_only_info_label, 5, 2, 1, 1)
        self.max_workers_spin = QtWidgets.QSpinBox(InstallDialog)
        self.max_workers_spin.setObjectName("max_workers_spin")
        self.install_dialog_layout.addWidget(self.max_workers_spin, 2, 1, 1, 1)
        self.ignore_space_check = QtWidgets.QCheckBox(InstallDialog)
        self.ignore_space_check.setObjectName("ignore_space_check")
        self.install_dialog_layout.addWidget(self.ignore_space_check, 4, 1, 1, 1)
        self.download_only_check = QtWidgets.QCheckBox(InstallDialog)
        self.download_only_check.setText("")
        self.download_only_check.setObjectName("download_only_check")
        self.install_dialog_layout.addWidget(self.download_only_check, 5, 1, 1, 1)
        self.max_workers_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_workers_info_label.setFont(font)
        self.max_workers_info_label.setObjectName("max_workers_info_label")
        self.install_dialog_layout.addWidget(self.max_workers_info_label, 2, 2, 1, 1)
        self.warn_message = QtWidgets.QLabel(InstallDialog)
        self.warn_message.setWordWrap(True)
        self.warn_message.setObjectName("warn_message")
        self.install_dialog_layout.addWidget(self.warn_message, 9, 1, 1, 2)
        self.ignore_space_label = QtWidgets.QLabel(InstallDialog)
        self.ignore_space_label.setObjectName("ignore_space_label")
        self.install_dialog_layout.addWidget(self.ignore_space_label, 4, 0, 1, 1, QtCore.Qt.AlignRight)
        self.install_dir_layout = QtWidgets.QHBoxLayout()
        self.install_dir_layout.setObjectName("install_dir_layout")
        self.install_dialog_layout.addLayout(self.install_dir_layout, 1, 1, 1, 2)
        self.download_size_label = QtWidgets.QLabel(InstallDialog)
        self.download_size_label.setObjectName("download_size_label")
        self.install_dialog_layout.addWidget(self.download_size_label, 7, 0, 1, 1, QtCore.Qt.AlignRight)
        self.force_download_label = QtWidgets.QLabel(InstallDialog)
        self.force_download_label.setObjectName("force_download_label")
        self.install_dialog_layout.addWidget(self.force_download_label, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.ignore_space_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.ignore_space_info_label.setFont(font)
        self.ignore_space_info_label.setObjectName("ignore_space_info_label")
        self.install_dialog_layout.addWidget(self.ignore_space_info_label, 4, 2, 1, 1)
        self.max_workers_label = QtWidgets.QLabel(InstallDialog)
        self.max_workers_label.setObjectName("max_workers_label")
        self.install_dialog_layout.addWidget(self.max_workers_label, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.download_size_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_size_info_label.setFont(font)
        self.download_size_info_label.setObjectName("download_size_info_label")
        self.install_dialog_layout.addWidget(self.download_size_info_label, 7, 1, 1, 2)
        self.button_layout = QtWidgets.QHBoxLayout()
        self.button_layout.setObjectName("button_layout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.button_layout.addItem(spacerItem1)
        self.cancel_button = QtWidgets.QPushButton(InstallDialog)
        self.cancel_button.setObjectName("cancel_button")
        self.button_layout.addWidget(self.cancel_button)
        self.verify_button = QtWidgets.QPushButton(InstallDialog)
        self.verify_button.setObjectName("verify_button")
        self.button_layout.addWidget(self.verify_button)
        self.install_button = QtWidgets.QPushButton(InstallDialog)
        self.install_button.setObjectName("install_button")
        self.button_layout.addWidget(self.install_button)
        self.install_dialog_layout.addLayout(self.button_layout, 11, 0, 1, 3)
        self.sdl_list_frame = QtWidgets.QFrame(InstallDialog)
        self.sdl_list_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.sdl_list_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.sdl_list_frame.setObjectName("sdl_list_frame")
        self.sdl_list_layout = QtWidgets.QVBoxLayout(self.sdl_list_frame)
        self.sdl_list_layout.setSpacing(0)
        self.sdl_list_layout.setObjectName("sdl_list_layout")
        self.install_dialog_layout.addWidget(self.sdl_list_frame, 6, 1, 1, 2, QtCore.Qt.AlignTop)
        self.warn_label = QtWidgets.QLabel(InstallDialog)
        self.warn_label.setObjectName("warn_label")
        self.install_dialog_layout.addWidget(self.warn_label, 9, 0, 1, 1, QtCore.Qt.AlignRight)

        self.retranslateUi(InstallDialog)
        QtCore.QMetaObject.connectSlotsByName(InstallDialog)

    def retranslateUi(self, InstallDialog):
        _translate = QtCore.QCoreApplication.translate
        self.install_dir_label.setText(_translate("InstallDialog", "Install directory"))
        self.download_only_label.setText(_translate("InstallDialog", "Download only"))
        self.sdl_list_label.setText(_translate("InstallDialog", "Optional packs"))
        self.install_size_info_label.setText(_translate("InstallDialog", "Click verify..."))
        self.install_dialog_label.setText(_translate("InstallDialog", "error"))
        self.install_size_label.setText(_translate("InstallDialog", "Total install size"))
        self.download_only_info_label.setText(_translate("InstallDialog", "Do not try to install."))
        self.max_workers_info_label.setText(_translate("InstallDialog", "Less is slower. (0: Default)"))
        self.warn_message.setText(_translate("InstallDialog", "TextLabel"))
        self.ignore_space_label.setText(_translate("InstallDialog", "Ignore free space"))
        self.download_size_label.setText(_translate("InstallDialog", "Download size"))
        self.force_download_label.setText(_translate("InstallDialog", "Force redownload"))
        self.ignore_space_info_label.setText(_translate("InstallDialog", "Use with caution!"))
        self.max_workers_label.setText(_translate("InstallDialog", "Max workers"))
        self.download_size_info_label.setText(_translate("InstallDialog", "Click verify..."))
        self.cancel_button.setText(_translate("InstallDialog", "Cancel"))
        self.verify_button.setText(_translate("InstallDialog", "Verify"))
        self.install_button.setText(_translate("InstallDialog", "Install"))
        self.warn_label.setText(_translate("InstallDialog", "Warnings"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    InstallDialog = QtWidgets.QDialog()
    ui = Ui_InstallDialog()
    ui.setupUi(InstallDialog)
    InstallDialog.show()
    sys.exit(app.exec_())
