# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'about.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_About(object):
    def setupUi(self, About):
        About.setObjectName("About")
        About.resize(499, 396)
        self.gridLayout = QtWidgets.QGridLayout(About)
        self.gridLayout.setObjectName("gridLayout")
        self.open_browser = QtWidgets.QPushButton(About)
        self.open_browser.setObjectName("open_browser")
        self.gridLayout.addWidget(self.open_browser, 4, 1, 1, 1)
        self.version_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.version_label.setFont(font)
        self.version_label.setObjectName("version_label")
        self.gridLayout.addWidget(self.version_label, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.license_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.license_label.setFont(font)
        self.license_label.setObjectName("license_label")
        self.gridLayout.addWidget(self.license_label, 7, 0, 1, 1, QtCore.Qt.AlignRight)
        self.lgd_dev_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lgd_dev_label.setFont(font)
        self.lgd_dev_label.setObjectName("lgd_dev_label")
        self.gridLayout.addWidget(self.lgd_dev_label, 6, 0, 1, 1, QtCore.Qt.AlignRight)
        self.dev = QtWidgets.QLabel(About)
        self.dev.setText("<a href=\'https://github.com/Dummerle\'>Dummerle</a>")
        self.dev.setOpenExternalLinks(True)
        self.dev.setObjectName("dev")
        self.gridLayout.addWidget(self.dev, 5, 1, 1, 1)
        self.update_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.update_label.setFont(font)
        self.update_label.setObjectName("update_label")
        self.gridLayout.addWidget(self.update_label, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.version = QtWidgets.QLabel(About)
        self.version.setText("error")
        self.version.setObjectName("version")
        self.gridLayout.addWidget(self.version, 2, 1, 1, 1)
        self.dev_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dev_label.setFont(font)
        self.dev_label.setObjectName("dev_label")
        self.gridLayout.addWidget(self.dev_label, 5, 0, 1, 1, QtCore.Qt.AlignRight)
        self.license = QtWidgets.QLabel(About)
        self.license.setObjectName("license")
        self.gridLayout.addWidget(self.license, 7, 1, 1, 1)
        self.update = QtWidgets.QLabel(About)
        self.update.setText("error")
        self.update.setObjectName("update")
        self.gridLayout.addWidget(self.update, 3, 1, 1, 1)
        self.lgd_dev = QtWidgets.QLabel(About)
        self.lgd_dev.setText("<a href=\'https://github.com/derrod/\'>derrod</a>")
        self.lgd_dev.setOpenExternalLinks(True)
        self.lgd_dev.setObjectName("lgd_dev")
        self.gridLayout.addWidget(self.lgd_dev, 6, 1, 1, 1)
        self.info_text = QtWidgets.QLabel(About)
        self.info_text.setWordWrap(True)
        self.info_text.setOpenExternalLinks(True)
        self.info_text.setObjectName("info_text")
        self.gridLayout.addWidget(self.info_text, 14, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 15, 0, 1, 3)
        self.title = QtWidgets.QLabel(About)
        self.title.setObjectName("title")
        self.gridLayout.addWidget(self.title, 1, 0, 1, 3)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 2, 2, 6, 1)

        self.retranslateUi(About)
        QtCore.QMetaObject.connectSlotsByName(About)

    def retranslateUi(self, About):
        _translate = QtCore.QCoreApplication.translate
        About.setWindowTitle(_translate("About", "About"))
        self.open_browser.setText(_translate("About", "Download latest release"))
        self.version_label.setText(_translate("About", "Version"))
        self.license_label.setText(_translate("About", "License"))
        self.lgd_dev_label.setText(_translate("About", "Legendary Developer"))
        self.dev.setToolTip(_translate("About", "Github"))
        self.update_label.setText(_translate("About", "Update available"))
        self.dev_label.setText(_translate("About", "Rare Developer"))
        self.license.setText(_translate("About", "GNU General Public License v3.0"))
        self.lgd_dev.setToolTip(_translate("About", "Github"))
        self.info_text.setText(_translate("About",
                                          "This is a beta version, so you can get bugs. If you get a bug, please report it by creating an Issue on <a href=\'https://github.com/Dummerle/Rare/issues\'>Github</a>. You can also contact me on Discord (Dummerle#7419). Or you can join the <a href=\'https://discord.gg/YvmABK9YSk\'>Discord server</a>"))
        self.title.setText(_translate("About", "<h2>About</h2>"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    About = QtWidgets.QWidget()
    ui = Ui_About()
    ui.setupUi(About)
    About.show()
    sys.exit(app.exec_())
