# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'game_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_GameSettings(object):
    def setupUi(self, GameSettings):
        GameSettings.setObjectName("GameSettings")
        GameSettings.resize(759, 617)
        self.game_settings_layout = QtWidgets.QVBoxLayout(GameSettings)
        self.game_settings_layout.setObjectName("game_settings_layout")
        self.title = QtWidgets.QLabel(GameSettings)
        self.title.setText("error")
        self.title.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse | QtCore.Qt.TextSelectableByMouse)
        self.title.setObjectName("title")
        self.game_settings_layout.addWidget(self.title)
        self.launch_settings_groupbox = QtWidgets.QGroupBox(GameSettings)
        self.launch_settings_groupbox.setObjectName("launch_settings_groupbox")
        self.launch_settings_layout = QtWidgets.QGridLayout(self.launch_settings_groupbox)
        self.launch_settings_layout.setObjectName("launch_settings_layout")
        self.skip_update = QtWidgets.QComboBox(self.launch_settings_groupbox)
        self.skip_update.setObjectName("skip_update")
        self.skip_update.addItem("")
        self.skip_update.addItem("")
        self.skip_update.addItem("")
        self.launch_settings_layout.addWidget(self.skip_update, 0, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.cloud_sync = QtWidgets.QCheckBox(self.launch_settings_groupbox)
        self.cloud_sync.setText("")
        self.cloud_sync.setObjectName("cloud_sync")
        self.launch_settings_layout.addWidget(self.cloud_sync, 2, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.launch_params_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.launch_params_label.setObjectName("launch_params_label")
        self.launch_settings_layout.addWidget(self.launch_params_label, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.wrapper_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.wrapper_label.setObjectName("wrapper_label")
        self.launch_settings_layout.addWidget(self.wrapper_label, 4, 0, 1, 1, QtCore.Qt.AlignRight)
        self.skip_update_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.skip_update_label.setObjectName("skip_update_label")
        self.launch_settings_layout.addWidget(self.skip_update_label, 0, 0, 1, 1, QtCore.Qt.AlignRight)
        self.wrapper_widget = QtWidgets.QWidget(self.launch_settings_groupbox)
        self.wrapper_widget.setObjectName("wrapper_widget")
        self.wrapper_layout = QtWidgets.QHBoxLayout(self.wrapper_widget)
        self.wrapper_layout.setContentsMargins(0, 0, 0, 0)
        self.wrapper_layout.setObjectName("wrapper_layout")
        self.wrapper = QtWidgets.QLineEdit(self.wrapper_widget)
        self.wrapper.setMinimumSize(QtCore.QSize(400, 0))
        self.wrapper.setObjectName("wrapper")
        self.wrapper_layout.addWidget(self.wrapper)
        self.wrapper_button = QtWidgets.QPushButton(self.wrapper_widget)
        self.wrapper_button.setObjectName("wrapper_button")
        self.wrapper_layout.addWidget(self.wrapper_button)
        self.launch_settings_layout.addWidget(self.wrapper_widget, 4, 1, 1, 1)
        self.offline_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.offline_label.setObjectName("offline_label")
        self.launch_settings_layout.addWidget(self.offline_label, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        self.offline = QtWidgets.QComboBox(self.launch_settings_groupbox)
        self.offline.setObjectName("offline")
        self.offline.addItem("")
        self.offline.addItem("")
        self.offline.addItem("")
        self.launch_settings_layout.addWidget(self.offline, 1, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.launch_params_widget = QtWidgets.QWidget(self.launch_settings_groupbox)
        self.launch_params_widget.setObjectName("launch_params_widget")
        self.launch_params_layout = QtWidgets.QHBoxLayout(self.launch_params_widget)
        self.launch_params_layout.setContentsMargins(0, 0, 0, 0)
        self.launch_params_layout.setObjectName("launch_params_layout")
        self.launch_params = QtWidgets.QLineEdit(self.launch_params_widget)
        self.launch_params.setMinimumSize(QtCore.QSize(400, 0))
        self.launch_params.setObjectName("launch_params")
        self.launch_params_layout.addWidget(self.launch_params)
        self.launch_params_button = QtWidgets.QPushButton(self.launch_params_widget)
        self.launch_params_button.setObjectName("launch_params_button")
        self.launch_params_layout.addWidget(self.launch_params_button)
        self.launch_settings_layout.addWidget(self.launch_params_widget, 3, 1, 1, 1)
        self.cloud_sync_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.cloud_sync_label.setObjectName("cloud_sync_label")
        self.launch_settings_layout.addWidget(self.cloud_sync_label, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.game_settings_layout.addWidget(self.launch_settings_groupbox)
        self.proton_groupbox = QtWidgets.QGroupBox(GameSettings)
        self.proton_groupbox.setObjectName("proton_groupbox")
        self.proton_layout = QtWidgets.QGridLayout(self.proton_groupbox)
        self.proton_layout.setObjectName("proton_layout")
        self.proton_wrapper = QtWidgets.QComboBox(self.proton_groupbox)
        self.proton_wrapper.setObjectName("proton_wrapper")
        self.proton_wrapper.addItem("")
        self.proton_layout.addWidget(self.proton_wrapper, 0, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.proton_wrapper_label = QtWidgets.QLabel(self.proton_groupbox)
        self.proton_wrapper_label.setObjectName("proton_wrapper_label")
        self.proton_layout.addWidget(self.proton_wrapper_label, 0, 0, 1, 1, QtCore.Qt.AlignRight)
        self.proton_prefix_label = QtWidgets.QLabel(self.proton_groupbox)
        self.proton_prefix_label.setObjectName("proton_prefix_label")
        self.proton_layout.addWidget(self.proton_prefix_label, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        self.proton_layout.addItem(spacerItem, 4, 1, 1, 1)
        self.proton_prefix_layout = QtWidgets.QVBoxLayout()
        self.proton_prefix_layout.setObjectName("proton_prefix_layout")
        self.proton_layout.addLayout(self.proton_prefix_layout, 1, 1, 2, 1)
        self.proton_prefix_alignment_label = QtWidgets.QLabel(self.proton_groupbox)
        self.proton_prefix_alignment_label.setEnabled(False)
        self.proton_prefix_alignment_label.setText("")
        self.proton_prefix_alignment_label.setObjectName("proton_prefix_alignment_label")
        self.proton_layout.addWidget(self.proton_prefix_alignment_label, 2, 0, 1, 1)
        self.linux_layout = QtWidgets.QVBoxLayout()
        self.linux_layout.setSpacing(0)
        self.linux_layout.setObjectName("linux_layout")
        self.proton_layout.addLayout(self.linux_layout, 3, 0, 1, 2)
        self.game_settings_layout.addWidget(self.proton_groupbox)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.game_settings_layout.addItem(spacerItem1)

        self.retranslateUi(GameSettings)
        QtCore.QMetaObject.connectSlotsByName(GameSettings)

    def retranslateUi(self, GameSettings):
        _translate = QtCore.QCoreApplication.translate
        GameSettings.setWindowTitle(_translate("GameSettings", "GameSettings"))
        self.launch_settings_groupbox.setTitle(_translate("GameSettings", "Launch Settings"))
        self.skip_update.setItemText(0, _translate("GameSettings", "Default"))
        self.skip_update.setItemText(1, _translate("GameSettings", "Yes"))
        self.skip_update.setItemText(2, _translate("GameSettings", "No"))
        self.launch_params_label.setText(_translate("GameSettings", "Launch parameters"))
        self.wrapper_label.setText(_translate("GameSettings", "Wrapper"))
        self.skip_update_label.setText(_translate("GameSettings", "Skip update check"))
        self.wrapper.setPlaceholderText(_translate("GameSettings", "e.g. optirun"))
        self.wrapper_button.setText(_translate("GameSettings", "Save"))
        self.offline_label.setText(_translate("GameSettings", "Offline mode"))
        self.offline.setItemText(0, _translate("GameSettings", "Default"))
        self.offline.setItemText(1, _translate("GameSettings", "Yes"))
        self.offline.setItemText(2, _translate("GameSettings", "No"))
        self.launch_params.setPlaceholderText(_translate("GameSettings", "parameters"))
        self.launch_params_button.setText(_translate("GameSettings", "Save"))
        self.cloud_sync_label.setText(_translate("GameSettings", "Sync with cloud"))
        self.proton_groupbox.setTitle(_translate("GameSettings", "Linux Settings"))
        self.proton_wrapper.setItemText(0, _translate("GameSettings", "Don\'t use Proton"))
        self.proton_wrapper_label.setText(_translate("GameSettings", "Proton"))
        self.proton_prefix_label.setText(_translate("GameSettings", "Prefix"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    GameSettings = QtWidgets.QWidget()
    ui = Ui_GameSettings()
    ui.setupUi(GameSettings)
    GameSettings.show()
    sys.exit(app.exec_())
